@@ -2,6 +2,7 @@ package app
 
 import (
 	"context"
+	"crypto/tls"
 	"database/sql"
 	"fmt"
 	"net"
@@ -9,6 +10,7 @@ import (
 	"path/filepath"
 	"runtime"
 	"sync"
+	"sync/atomic"
 	"time"
 
 	"github.com/canonical/go-dqlite"
@@ -19,6 +21,10 @@ import (
 	"golang.org/x/sync/semaphore"
 )
 
+// used to create a unique driver name, MUST be modified atomically
+// https://pkg.go.dev/sync/atomic#AddInt64
+var driverIndex int64
+
 // App is a high-level helper for initializing a typical dqlite-based Go
 // application.
 //
@@ -187,6 +193,8 @@ func New(dir string, options ...Option) (app *App, err error) {
 		dqlite.WithFailureDomain(o.FailureDomain),
 		dqlite.WithNetworkLatency(o.NetworkLatency),
 		dqlite.WithSnapshotParams(o.SnapshotParams),
+		dqlite.WithDiskMode(o.DiskMode),
+		dqlite.WithAutoRecovery(o.AutoRecovery),
 	)
 	if err != nil {
 		stop()
@@ -206,13 +214,17 @@ func New(dir string, options ...Option) (app *App, err error) {
 		driverDial = o.Conn.dialFunc
 	}
 
-	driver, err := driver.New(store, driver.WithDialFunc(driverDial), driver.WithLogFunc(o.Log))
+	driver, err := driver.New(
+		store,
+		driver.WithDialFunc(driverDial),
+		driver.WithLogFunc(o.Log),
+		driver.WithTracing(o.Tracing),
+	)
 	if err != nil {
 		stop()
 		return nil, fmt.Errorf("create driver: %w", err)
 	}
-	driverIndex++
-	driverName := fmt.Sprintf("dqlite-%d", driverIndex)
+	driverName := fmt.Sprintf("dqlite-%d", atomic.AddInt64(&driverIndex, 1))
 	sql.Register(driverName, driver)
 
 	if o.Voters < 3 || o.Voters%2 == 0 {
@@ -271,12 +283,18 @@ func New(dir string, options ...Option) (app *App, err error) {
 			for {
 				remote := <-o.Conn.acceptCh
 
-				// Write the status line and upgrade header by hand since w.WriteHeader() would fail after Hijack().
-				data := []byte("HTTP/1.1 101 Switching Protocols\r\nUpgrade: dqlite\r\n\r\n")
-				n, err := remote.Write(data)
-				if err != nil || n != len(data) {
-					remote.Close()
-					panic(fmt.Errorf("failed to write connection header: %w", err))
+				// keep forward compatible
+				_, isTcp := remote.(*net.TCPConn)
+				_, isTLS := remote.(*tls.Conn)
+
+				if isTcp || isTLS {
+					// Write the status line and upgrade header by hand since w.WriteHeader() would fail after Hijack().
+					data := []byte("HTTP/1.1 101 Switching Protocols\r\nUpgrade: dqlite\r\n\r\n")
+					n, err := remote.Write(data)
+					if err != nil || n != len(data) {
+						remote.Close()
+						panic(fmt.Errorf("failed to write connection header: %w", err))
+					}
 				}
 
 				local, err := net.Dial("unix", nodeBindAddress)
@@ -304,6 +322,8 @@ func (a *App) Handover(ctx context.Context) error {
 	// Set a hard limit of one minute, in case the user-provided context
 	// has no expiration. That avoids the call to stop responding forever
 	// in case a majority of the cluster is down and no leader is available.
+	// Watch out when removing or editing this context, the for loop at the
+	// end of this function will possibly run "forever" without it.
 	var cancel context.CancelFunc
 	ctx, cancel = context.WithTimeout(ctx, time.Minute)
 	defer cancel()
@@ -350,7 +370,7 @@ func (a *App) Handover(ctx context.Context) error {
 			return fmt.Errorf("cluster servers: %w", err)
 		}
 		changes := a.makeRolesChanges(nodes)
-		voters := changes.list(client.Voter, true)
+		voters := changes.list(client.Voter, true, nil)
 
 		for i, voter := range voters {
 			if voter.Address == a.address {
@@ -373,8 +393,22 @@ func (a *App) Handover(ctx context.Context) error {
 
 	// Demote ourselves if we have promoted someone else.
 	if role != -1 {
-		if err := cli.Assign(ctx, a.ID(), client.Spare); err != nil {
-			return fmt.Errorf("demote ourselves: %w", err)
+		// Try a while before failing. The new leader has to possibly commit an entry
+		// from its new term in order to commit the last configuration change, wait a bit
+		// for that to happen and don't fail immediately
+		for {
+			err = cli.Assign(ctx, a.ID(), client.Spare)
+			if err == nil {
+				return nil
+			}
+			select {
+			case <-ctx.Done():
+				return fmt.Errorf("demote ourselves context done: %w", err)
+			default:
+				// Wait a bit before trying again
+				time.Sleep(time.Second)
+				continue
+			}
 		}
 	}
 
@@ -710,5 +744,3 @@ func (a *App) warn(format string, args ...interface{}) {
 func (a *App) error(format string, args ...interface{}) {
 	a.log(client.LogError, format, args...)
 }
-
-var driverIndex = 0
