@@ -2,6 +2,8 @@ package main
 
 import (
 	"context"
+	"crypto/tls"
+	"crypto/x509"
 	"fmt"
 	"io/ioutil"
 	"log"
@@ -25,6 +27,9 @@ func main() {
 	var join *[]string
 	var dir string
 	var verbose bool
+	var diskMode bool
+	var crt string
+	var key string
 
 	cmd := &cobra.Command{
 		Use:   "dqlite-demo",
@@ -43,7 +48,31 @@ Complete documentation is available at https://github.com/canonical/go-dqlite`,
 				}
 				log.Printf(fmt.Sprintf("%s: %s: %s\n", api, l.String(), format), a...)
 			}
-			app, err := app.New(dir, app.WithAddress(db), app.WithCluster(*join), app.WithLogFunc(logFunc))
+
+			options := []app.Option{app.WithAddress(db), app.WithCluster(*join), app.WithLogFunc(logFunc),
+				app.WithDiskMode(diskMode)}
+
+			// Set TLS options
+			if (crt != "" && key == "") || (key != "" && crt == "") {
+				return fmt.Errorf("both TLS certificate and key must be given")
+			}
+			if crt != "" {
+				cert, err := tls.LoadX509KeyPair(crt, key)
+				if err != nil {
+					return err
+				}
+				data, err := ioutil.ReadFile(crt)
+				if err != nil {
+					return err
+				}
+				pool := x509.NewCertPool()
+				if !pool.AppendCertsFromPEM(data) {
+					return fmt.Errorf("bad certificate")
+				}
+				options = append(options, app.WithTLS(app.SimpleTLSConfig(cert, pool)))
+			}
+
+			app, err := app.New(dir, options...)
 			if err != nil {
 				return err
 			}
@@ -91,7 +120,7 @@ Complete documentation is available at https://github.com/canonical/go-dqlite`,
 
 			go http.Serve(listener, nil)
 
-			ch := make(chan os.Signal)
+			ch := make(chan os.Signal, 32)
 			signal.Notify(ch, unix.SIGPWR)
 			signal.Notify(ch, unix.SIGINT)
 			signal.Notify(ch, unix.SIGQUIT)
@@ -115,6 +144,9 @@ Complete documentation is available at https://github.com/canonical/go-dqlite`,
 	join = flags.StringSliceP("join", "j", nil, "database addresses of existing nodes")
 	flags.StringVarP(&dir, "dir", "D", "/tmp/dqlite-demo", "data directory")
 	flags.BoolVarP(&verbose, "verbose", "v", false, "verbose logging")
+	flags.BoolVar(&diskMode, "disk", defaultDiskMode, "Warning: Unstable, Experimental. Set this flag to enable dqlite's disk-mode.")
+	flags.StringVarP(&crt, "cert", "c", "", "public TLS cert")
+	flags.StringVarP(&key, "key", "k", "", "private TLS key")
 
 	cmd.MarkFlagRequired("api")
 	cmd.MarkFlagRequired("db")
@@ -125,7 +157,8 @@ Complete documentation is available at https://github.com/canonical/go-dqlite`,
 }
 
 const (
-	schema = "CREATE TABLE IF NOT EXISTS model (key TEXT, value TEXT, UNIQUE(key))"
-	query  = "SELECT value FROM model WHERE key = ?"
-	update = "INSERT OR REPLACE INTO model(key, value) VALUES(?, ?)"
+	schema          = "CREATE TABLE IF NOT EXISTS model (key TEXT, value TEXT, UNIQUE(key))"
+	query           = "SELECT value FROM model WHERE key = ?"
+	update          = "INSERT OR REPLACE INTO model(key, value) VALUES(?, ?)"
+	defaultDiskMode = false
 )
