@@ -19,6 +19,7 @@ import (
 	"database/sql/driver"
 	"fmt"
 	"io"
+	"math"
 	"net"
 	"reflect"
 	"syscall"
@@ -28,6 +29,7 @@ import (
 
 	"github.com/canonical/go-dqlite/client"
 	"github.com/canonical/go-dqlite/internal/protocol"
+	"github.com/canonical/go-dqlite/tracing"
 )
 
 // Driver perform queries against a dqlite server.
@@ -60,9 +62,6 @@ const (
 	errIoErrLeadershipLostLegacy = errIoErr | (33 << 8)
 )
 
-// Max amount of parameters in a Tuple.
-const MaxTupleParams = 255
-
 // Option can be used to tweak driver parameters.
 type Option func(*options)
 
@@ -260,7 +259,7 @@ func (c *Connector) Connect(ctx context.Context) (driver.Conn, error) {
 	var err error
 	conn.protocol, err = connector.Connect(ctx)
 	if err != nil {
-		return nil, errors.Wrap(err, "failed to create dqlite connection")
+		return nil, driverError(conn.log, errors.Wrap(err, "failed to create dqlite connection"))
 	}
 
 	conn.request.Init(4096)
@@ -270,13 +269,13 @@ func (c *Connector) Connect(ctx context.Context) (driver.Conn, error) {
 
 	if err := conn.protocol.Call(ctx, &conn.request, &conn.response); err != nil {
 		conn.protocol.Close()
-		return nil, errors.Wrap(err, "failed to open database")
+		return nil, driverError(conn.log, errors.Wrap(err, "failed to open database"))
 	}
 
 	conn.id, err = protocol.DecodeDb(&conn.response)
 	if err != nil {
 		conn.protocol.Close()
-		return nil, errors.Wrap(err, "failed to open database")
+		return nil, driverError(conn.log, errors.Wrap(err, "failed to open database"))
 	}
 
 	return conn, nil
@@ -341,6 +340,9 @@ type Conn struct {
 // context is for the preparation of the statement, it must not store the
 // context within the statement itself.
 func (c *Conn) PrepareContext(ctx context.Context, query string) (driver.Stmt, error) {
+	ctx, span := tracing.Start(ctx, "dqlite.driver.PrepareContext", query)
+	defer span.End()
+
 	stmt := &Stmt{
 		protocol: c.protocol,
 		request:  &c.request,
@@ -351,11 +353,18 @@ func (c *Conn) PrepareContext(ctx context.Context, query string) (driver.Stmt, e
 
 	protocol.EncodePrepare(&c.request, uint64(c.id), query)
 
-	if err := c.protocol.Call(ctx, &c.request, &c.response); err != nil {
+	var start time.Time
+	if c.tracing != client.LogNone {
+		start = time.Now()
+	}
+	err := c.protocol.Call(ctx, &c.request, &c.response)
+	if c.tracing != client.LogNone {
+		c.log(c.tracing, "%.3fs request prepared: %q", time.Since(start).Seconds(), query)
+	}
+	if err != nil {
 		return nil, driverError(c.log, err)
 	}
 
-	var err error
 	stmt.db, stmt.id, stmt.params, err = protocol.DecodeStmt(&c.response)
 	if err != nil {
 		return nil, driverError(c.log, err)
@@ -375,23 +384,33 @@ func (c *Conn) Prepare(query string) (driver.Stmt, error) {
 
 // ExecContext is an optional interface that may be implemented by a Conn.
 func (c *Conn) ExecContext(ctx context.Context, query string, args []driver.NamedValue) (driver.Result, error) {
-	if len(args) > MaxTupleParams {
-		return nil, driverError(c.log, fmt.Errorf("too many parameters (%d) max = %d", len(args), MaxTupleParams))
+	ctx, span := tracing.Start(ctx, "dqlite.driver.ExecContext", query)
+	defer span.End()
+
+	if int64(len(args)) > math.MaxUint32 {
+		return nil, driverError(c.log, fmt.Errorf("too many parameters (%d)", len(args)))
+	} else if len(args) > math.MaxUint8 {
+		protocol.EncodeExecSQLV1(&c.request, uint64(c.id), query, args)
+	} else {
+		protocol.EncodeExecSQLV0(&c.request, uint64(c.id), query, args)
 	}
 
-	protocol.EncodeExecSQL(&c.request, uint64(c.id), query, args)
-
-	if err := c.protocol.Call(ctx, &c.request, &c.response); err != nil {
-		return nil, driverError(c.log, err)
+	var start time.Time
+	if c.tracing != client.LogNone {
+		start = time.Now()
+	}
+	err := c.protocol.Call(ctx, &c.request, &c.response)
+	if c.tracing != client.LogNone {
+		c.log(c.tracing, "%.3fs request exec: %q", time.Since(start).Seconds(), query)
 	}
-
-	result, err := protocol.DecodeResult(&c.response)
 	if err != nil {
 		return nil, driverError(c.log, err)
 	}
 
-	if c.tracing != client.LogNone {
-		c.log(c.tracing, "exec: %s", query)
+	var result protocol.Result
+	result, err = protocol.DecodeResult(&c.response)
+	if err != nil {
+		return nil, driverError(c.log, err)
 	}
 
 	return &Result{result: result}, nil
@@ -404,23 +423,33 @@ func (c *Conn) Query(query string, args []driver.Value) (driver.Rows, error) {
 
 // QueryContext is an optional interface that may be implemented by a Conn.
 func (c *Conn) QueryContext(ctx context.Context, query string, args []driver.NamedValue) (driver.Rows, error) {
-	if len(args) > MaxTupleParams {
-		return nil, driverError(c.log, fmt.Errorf("too many parameters (%d) max = %d", len(args), MaxTupleParams))
+	ctx, span := tracing.Start(ctx, "dqlite.driver.QueryContext", query)
+	defer span.End()
+
+	if int64(len(args)) > math.MaxUint32 {
+		return nil, driverError(c.log, fmt.Errorf("too many parameters (%d)", len(args)))
+	} else if len(args) > math.MaxUint8 {
+		protocol.EncodeQuerySQLV1(&c.request, uint64(c.id), query, args)
+	} else {
+		protocol.EncodeQuerySQLV0(&c.request, uint64(c.id), query, args)
 	}
 
-	protocol.EncodeQuerySQL(&c.request, uint64(c.id), query, args)
-
-	if err := c.protocol.Call(ctx, &c.request, &c.response); err != nil {
-		return nil, driverError(c.log, err)
+	var start time.Time
+	if c.tracing != client.LogNone {
+		start = time.Now()
+	}
+	err := c.protocol.Call(ctx, &c.request, &c.response)
+	if c.tracing != client.LogNone {
+		c.log(c.tracing, "%.3fs request query: %q", time.Since(start).Seconds(), query)
 	}
-
-	rows, err := protocol.DecodeRows(&c.response)
 	if err != nil {
 		return nil, driverError(c.log, err)
 	}
 
-	if c.tracing != client.LogNone {
-		c.log(c.tracing, "query: %s", query)
+	var rows protocol.Rows
+	rows, err = protocol.DecodeRows(&c.response)
+	if err != nil {
+		return nil, driverError(c.log, err)
 	}
 
 	return &Rows{
@@ -557,23 +586,33 @@ func (s *Stmt) NumInput() int {
 //
 // ExecContext must honor the context timeout and return when it is canceled.
 func (s *Stmt) ExecContext(ctx context.Context, args []driver.NamedValue) (driver.Result, error) {
-	if len(args) > MaxTupleParams {
-		return nil, driverError(s.log, fmt.Errorf("too many parameters (%d) max = %d", len(args), MaxTupleParams))
+	ctx, span := tracing.Start(ctx, "dqlite.driver.Stmt.ExecContext", s.sql)
+	defer span.End()
+
+	if int64(len(args)) > math.MaxUint32 {
+		return nil, driverError(s.log, fmt.Errorf("too many parameters (%d)", len(args)))
+	} else if len(args) > math.MaxUint8 {
+		protocol.EncodeExecV1(s.request, s.db, s.id, args)
+	} else {
+		protocol.EncodeExecV0(s.request, s.db, s.id, args)
 	}
 
-	protocol.EncodeExec(s.request, s.db, s.id, args)
-
-	if err := s.protocol.Call(ctx, s.request, s.response); err != nil {
-		return nil, driverError(s.log, err)
+	var start time.Time
+	if s.tracing != client.LogNone {
+		start = time.Now()
+	}
+	err := s.protocol.Call(ctx, s.request, s.response)
+	if s.tracing != client.LogNone {
+		s.log(s.tracing, "%.3fs request prepared: %q", time.Since(start).Seconds(), s.sql)
 	}
-
-	result, err := protocol.DecodeResult(s.response)
 	if err != nil {
 		return nil, driverError(s.log, err)
 	}
 
-	if s.tracing != client.LogNone {
-		s.log(s.tracing, "exec prepared: %s", s.sql)
+	var result protocol.Result
+	result, err = protocol.DecodeResult(s.response)
+	if err != nil {
+		return nil, driverError(s.log, err)
 	}
 
 	return &Result{result: result}, nil
@@ -589,23 +628,33 @@ func (s *Stmt) Exec(args []driver.Value) (driver.Result, error) {
 //
 // QueryContext must honor the context timeout and return when it is canceled.
 func (s *Stmt) QueryContext(ctx context.Context, args []driver.NamedValue) (driver.Rows, error) {
-	if len(args) > MaxTupleParams {
-		return nil, driverError(s.log, fmt.Errorf("too many parameters (%d) max = %d", len(args), MaxTupleParams))
+	ctx, span := tracing.Start(ctx, "dqlite.driver.Stmt.QueryContext", s.sql)
+	defer span.End()
+
+	if int64(len(args)) > math.MaxUint32 {
+		return nil, driverError(s.log, fmt.Errorf("too many parameters (%d)", len(args)))
+	} else if len(args) > math.MaxUint8 {
+		protocol.EncodeQueryV1(s.request, s.db, s.id, args)
+	} else {
+		protocol.EncodeQueryV0(s.request, s.db, s.id, args)
 	}
 
-	protocol.EncodeQuery(s.request, s.db, s.id, args)
-
-	if err := s.protocol.Call(ctx, s.request, s.response); err != nil {
-		return nil, driverError(s.log, err)
+	var start time.Time
+	if s.tracing != client.LogNone {
+		start = time.Now()
+	}
+	err := s.protocol.Call(ctx, s.request, s.response)
+	if s.tracing != client.LogNone {
+		s.log(s.tracing, "%.3fs request prepared: %q", time.Since(start).Seconds(), s.sql)
 	}
-
-	rows, err := protocol.DecodeRows(s.response)
 	if err != nil {
 		return nil, driverError(s.log, err)
 	}
 
-	if s.tracing != client.LogNone {
-		s.log(s.tracing, "query prepared: %s", s.sql)
+	var rows protocol.Rows
+	rows, err = protocol.DecodeRows(s.response)
+	if err != nil {
+		return nil, driverError(s.log, err)
 	}
 
 	return &Rows{ctx: ctx, request: s.request, response: s.response, protocol: s.protocol, rows: rows}, nil
@@ -754,6 +803,12 @@ type unwrappable interface {
 	Unwrap() error
 }
 
+// TODO driver.ErrBadConn should not be returned when there's a possibility that
+// the query has been executed. In our case there is a window in protocol.Call
+// between `send` and `recv` where the send has succeeded but the recv has
+// failed. In those cases we call driverError on the result of protocol.Call,
+// possibly returning ErrBadCon.
+// https://cs.opensource.google/go/go/+/refs/tags/go1.20.4:src/database/sql/driver/driver.go;drc=a32a592c8c14927c20ac42808e1fb2e55b2e9470;l=162
 func driverError(log client.LogFunc, err error) error {
 	switch err := errors.Cause(err).(type) {
 	case syscall.Errno:
@@ -804,5 +859,9 @@ func driverError(log client.LogFunc, err error) error {
 			return driver.ErrBadConn
 		}
 	}
+	if errors.Is(err, io.EOF) {
+		log(client.LogDebug, "EOF detected: %v", err)
+		return driver.ErrBadConn
+	}
 	return err
 }
