@@ -69,6 +69,8 @@ func (s *Shell) Process(ctx context.Context, line string) (string, error) {
 		return s.processCluster(ctx, line)
 	case ".leader":
 		return s.processLeader(ctx, line)
+	case ".help":
+		return s.processHelp(), nil
 	}
 	if strings.HasPrefix(strings.ToLower(strings.TrimLeft(line, " ")), ".remove") {
 		return s.processRemove(ctx, line)
@@ -85,11 +87,22 @@ func (s *Shell) Process(ctx context.Context, line string) (string, error) {
 	if strings.HasPrefix(strings.ToLower(strings.TrimLeft(line, " ")), ".reconfigure") {
 		return s.processReconfigure(ctx, line)
 	}
-	if strings.HasPrefix(strings.ToUpper(strings.TrimLeft(line, " ")), "SELECT") {
-		return s.processSelect(ctx, line)
-	} else {
-		return "", s.processExec(ctx, line)
-	}
+	return s.processQuery(ctx, line)
+}
+
+func (s *Shell) processHelp() string {
+	return `
+Dqlite shell is a simple interactive prompt for inspecting a dqlite database.
+Enter a SQL statement to execute it, or one of the following built-in commands:
+
+  .cluster                          Show the cluster membership
+  .leader                           Show the current leader
+  .remove <address>                 Remove a node from the cluster
+  .describe <address>               Show the details of a node
+  .weight <address> <weight>        Set the weight of a node
+  .dump <address> [<database>]      Dump the database
+  .reconfigure <dir> <clusteryaml>  Reconfigure the cluster
+`[1:]
 }
 
 func (s *Shell) processCluster(ctx context.Context, line string) (string, error) {
@@ -300,7 +313,7 @@ func (s *Shell) processWeight(ctx context.Context, line string) (string, error)
 	return "", nil
 }
 
-func (s *Shell) processSelect(ctx context.Context, line string) (string, error) {
+func (s *Shell) processQuery(ctx context.Context, line string) (string, error) {
 	tx, err := s.db.BeginTx(ctx, nil)
 	if err != nil {
 		return "", fmt.Errorf("begin transaction: %w", err)
