@@ -69,6 +69,31 @@ func WithSnapshotParams(params SnapshotParams) Option {
 	}
 }
 
+// WithDiskMode enables dqlite disk-mode on the node.
+// WARNING: This is experimental API, use with caution
+// and prepare for data loss.
+// UNSTABLE: Behavior can change in future.
+// NOT RECOMMENDED for production use-cases, use at own risk.
+func WithDiskMode(disk bool) Option {
+	return func(options *options) {
+		options.DiskMode = disk
+	}
+}
+
+// WithAutoRecovery enables or disables auto-recovery of persisted data
+// at startup for this node.
+//
+// When auto-recovery is enabled, raft snapshots and segment files may be
+// deleted at startup if they are determined to be corrupt. This helps
+// the startup process to succeed in more cases, but can lead to data loss.
+//
+// Auto-recovery is enabled by default.
+func WithAutoRecovery(recovery bool) Option {
+	return func(options *options) {
+		options.AutoRecovery = recovery
+	}
+}
+
 // New creates a new Node instance.
 func New(id uint64, address string, dir string, options ...Option) (*Node, error) {
 	o := defaultOptions()
@@ -114,6 +139,18 @@ func New(id uint64, address string, dir string, options ...Option) (*Node, error
 			return nil, err
 		}
 	}
+	if o.DiskMode {
+		if err := server.EnableDiskMode(); err != nil {
+			cancel()
+			return nil, err
+		}
+	}
+	if !o.AutoRecovery {
+		if err := server.SetAutoRecovery(false); err != nil {
+			cancel()
+			return nil, err
+		}
+	}
 
 	s := &Node{
 		server:      server,
@@ -153,6 +190,8 @@ type options struct {
 	NetworkLatency uint64
 	FailureDomain  uint64
 	SnapshotParams bindings.SnapshotParams
+	DiskMode       bool
+	AutoRecovery   bool
 }
 
 // Close the server, releasing all resources it created.
@@ -213,5 +252,7 @@ func ReconfigureMembershipExt(dir string, cluster []NodeInfo) error {
 func defaultOptions() *options {
 	return &options{
 		DialFunc: client.DefaultDialFunc,
+		DiskMode: false, // Be explicit about not enabling disk-mode by default.
+		AutoRecovery: true,
 	}
 }
